/****************************************************************
 * Project name:
   MULTI TIMERS BY EVENT DRIVEN PROGRAMMING
 
 * Copyright:
    (c) Stevan Tosic
 
 * Email:
    stevantosic@yahoo.com
 
* Description:
   In this project there are three timers operating on 1,2 an 4 
   seconds respectively. Having expired a timer timeout, a pin
   on PORTB toggles his value (flashing):
   Timer0 : 1 sec : PORTB.F0
   Timer1 : 2 sec : PORTB.F1
   Timer2 : 4 sec : PORTB.F2.
   Number of possible timers is eight. Generally it depends on RAM memory and system setting and
   can be greater.

 * Resources:   
   1. Development board: BigPIC5
   2. Microcontroller: PIC 18F8520
   3. Compiler:  mikroC PRO for PIC 4.60.0.0
******************************************************************/

#include "edt.h"

int T0, T1, T2;

// init 10ms timer interrupt
void TimersInit(void)
{
    T0CON=0xC6;
    TMR0L = 61;
    INTCON = 0xA0;
    INTCON.PEIE = 1;
}

void interrupt (void)
{
   static unsigned int cnt=0;
   if(INTCON.TMR0IF)
   {
      cnt++;
      // every 1s call dispatcher
      if(cnt == 100)
      {
         TimerForDispatcher();
         cnt=0;
      }
      TMR0L = 61;
      INTCON.T0IF =0 ;
   }
}

void T0_tick(unsigned char t, void* ev_arg)
{
    PORTC.F0 = ~PORTC.F0;
}

void T1_tick(unsigned char t, void* ev_arg)
{
    PORTC.F1 = ~PORTC.F1;
}

void T2_tick(unsigned char t, void* ev_arg)
{
    PORTC.F2 = ~PORTC.F2;
}

void main() {
   TRISB = 0x00; TRISC = 0x00;
   PORTB = 0x00; PORTC = 0x00;

   EventsInit();
   TimersInit();
   T0 = TimerCreate(T0_tick);
   T1 = TimerCreate(T1_tick);
   T2 = TimerCreate(T2_tick);
   
   TimerStart(T0, 1);
   TimerStart(T1, 2);
   TimerStart(T2, 4);

   EventDispatcher();
}